@@ -407,7 +407,7 @@ gdk_window_new (GdkWindow     *parent,
 	  (private->colormap != gdk_colormap_get_system ()) &&
 	  (private->colormap != gdk_window_get_colormap (gdk_window_get_toplevel (window))))
 	{
-	  GDK_NOTE (MISC, g_print ("adding colormap window\n"));
+	  GDK_NOTE (MISC, g_message ("adding colormap window\n"));
 	  gdk_window_add_colormap_windows (window);
 	}
 
@@ -437,7 +437,7 @@ gdk_window_new (GdkWindow     *parent,
   if (attributes_mask & GDK_WA_TITLE)
     title = attributes->title;
   else
-    title = gdk_progname;
+    title = g_get_prgname ();
 
   XmbSetWMProperties (private->xdisplay, private->xwindow,
                       title, title,
@@ -704,8 +704,10 @@ gdk_window_unref (GdkWindow *window)
     {
       if (!private->destroyed)
 	g_warning ("losing last reference to undestroyed window\n");
+      g_dataset_destroy (window);
       g_free (window);
     }
+
 }
 
 void
@@ -1771,7 +1773,7 @@ gdk_window_dnd_data_set (GdkWindow       *window,
 		       data_numbytes);
   tmp = gdk_atom_name(window_private->dnd_drag_data_type);
 #ifdef DEBUG_DND
-  g_print("DnD type %s on window %ld\n", tmp, window_private->xwindow);
+  g_message("DnD type %s on window %ld\n", tmp, window_private->xwindow);
 #endif
   g_free(tmp);
   
@@ -1801,9 +1803,8 @@ gdk_window_dnd_data_set (GdkWindow       *window,
 
   if (!gdk_send_xevent (event->dragrequest.requestor, False,
 		       StructureNotifyMask, &sev))
-    GDK_NOTE (DND, g_print("Sending XdeDataAvailable to %#x failed\n",
-			   event->dragrequest.requestor));
-
+    GDK_NOTE (DND, g_message("Sending XdeDataAvailable to %#x failed\n",
+			     event->dragrequest.requestor));
 }
 
 void          
@@ -2077,3 +2078,11 @@ gdk_window_get_toplevels (void)
   return new_list;
 }
 
+void          
+gdk_drawable_set_data (GdkDrawable   *drawable,
+		       const gchar   *key,
+		       gpointer	      data,
+		       GDestroyNotify destroy_func)
+{
+  g_dataset_set_data_full (drawable, key, data, destroy_func);
+}
