@@ -71,7 +71,8 @@ struct _GtkContainerClass
   void (* remove)      		(GtkContainer	 *container,
 				 GtkWidget	 *widget);
   void (* check_resize)		(GtkContainer	 *container);
-  void (* foreach)     		(GtkContainer	 *container,
+  void (* forall)     		(GtkContainer	 *container,
+				 gboolean	  include_internals,
 				 GtkCallback	  callback,
 				 gpointer	  callbabck_data);
   gint (* focus)       		(GtkContainer	 *container,
@@ -174,6 +175,7 @@ void    gtk_container_child_setv   	   (GtkContainer      *container,
  */
 void    gtk_container_add_with_args	   (GtkContainer      *container,
 					    GtkWidget	      *widget,
+					    const gchar	      *first_arg_name,
 					    ...);
 void    gtk_container_addv		   (GtkContainer      *container,
 					    GtkWidget	      *widget,
@@ -181,6 +183,7 @@ void    gtk_container_addv		   (GtkContainer      *container,
 					    GtkArg	      *args);
 void	gtk_container_child_set		   (GtkContainer      *container,
 					    GtkWidget         *child,
+					    const gchar	      *first_arg_name,
 					    ...);
      
 
@@ -199,10 +202,14 @@ void    gtk_container_arg_get		   (GtkContainer *container,
 gchar*	gtk_container_child_args_collect   (GtkType       object_type,
 					    GSList      **arg_list_p,
 					    GSList      **info_list_p,
-					    gpointer      var_args_p);
+					    const gchar	 *first_arg_name,
+					    va_list       args);
 gchar*  gtk_container_child_arg_get_info   (GtkType       object_type,
 					    const gchar  *arg_name,
 					    GtkArgInfo  **info_p);
+void    gtk_container_forall		   (GtkContainer *container,
+					    GtkCallback	  callback,
+					    gpointer	  callback_data);
 
 
 /* Deprecated methods */
