@@ -51,10 +51,10 @@ static gint gtk_curve_graph_events (GtkWidget     *widget,
 				    GtkCurve      *c);
 static void gtk_curve_size_graph   (GtkCurve      *curve);
 
-guint
+GtkType
 gtk_curve_get_type (void)
 {
-  static guint curve_type = 0;
+  static GtkType curve_type = 0;
 
   if (!curve_type)
     {
@@ -65,8 +65,9 @@ gtk_curve_get_type (void)
 	sizeof (GtkCurveClass),
 	(GtkClassInitFunc) gtk_curve_class_init,
 	(GtkObjectInitFunc) gtk_curve_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
       curve_type = gtk_type_unique (gtk_drawing_area_get_type (), &curve_info);
@@ -86,7 +87,7 @@ gtk_curve_class_init (GtkCurveClass *class)
   curve_type_changed_signal =
     gtk_signal_new ("curve_type_changed", GTK_RUN_FIRST, object_class->type,
 		    GTK_SIGNAL_OFFSET (GtkCurveClass, curve_type_changed),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+		    gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
   gtk_object_class_add_signals (object_class, &curve_type_changed_signal, 1);
 
   object_class->finalize = gtk_curve_finalize;
