@@ -26,12 +26,15 @@
 
 #ifdef __cplusplus
 extern "C" {
+#pragma }
 #endif /* __cplusplus */
 
 
-#define GTK_EDITABLE(obj)          GTK_CHECK_CAST (obj, gtk_editable_get_type (), GtkEditable)
-#define GTK_EDITABLE_CLASS(klass)  GTK_CHECK_CLASS_CAST (klass, gtk_editable_get_type (), GtkEditableClass)
-#define GTK_IS_EDITABLE(obj)       GTK_CHECK_TYPE (obj, gtk_editable_get_type ())
+#define GTK_TYPE_EDITABLE            (gtk_editable_get_type ())
+#define GTK_EDITABLE(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_EDITABLE, GtkEditable))
+#define GTK_EDITABLE_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_EDITABLE, GtkEditableClass))
+#define GTK_IS_EDITABLE(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_EDITABLE))
+#define GTK_IS_EDITABLE_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_EDITABLE))
 
 
 typedef struct _GtkEditable       GtkEditable;
@@ -57,7 +60,9 @@ struct _GtkEditable
 struct _GtkEditableClass
 {
   GtkWidgetClass parent_class;
-
+  
+  /* Signals for notification/filtering of changes */
+  void (* changed)      (GtkEditable    *editable);
   void (* insert_text)  (GtkEditable    *editable,
 			 const gchar    *text,
 			 gint            length,
@@ -65,6 +70,38 @@ struct _GtkEditableClass
   void (* delete_text)  (GtkEditable    *editable,
 			 gint            start_pos,
 			 gint            end_pos);
+
+  /* Bindings actions */
+  void (* activate)        (GtkEditable *editable);
+  void (* set_editable)    (GtkEditable *editable,
+			    gboolean	 is_editable);
+  void (* move_cursor)     (GtkEditable *editable,
+			    gint         x,
+			    gint         y);
+  void (* move_word)       (GtkEditable *editable,
+			    gint         n);
+  void (* move_page)       (GtkEditable *editable,
+			    gint         x,
+			    gint         y);
+  void (* move_to_row)     (GtkEditable *editable,
+			    gint         row);
+  void (* move_to_column)  (GtkEditable *editable,
+			    gint         row);
+  void (* kill_char)       (GtkEditable *editable,
+			    gint         direction);
+  void (* kill_word)       (GtkEditable *editable,
+			    gint         direction);
+  void (* kill_line)       (GtkEditable *editable,
+			    gint         direction);
+  void (* cut_clipboard)   (GtkEditable *editable);
+  void (* copy_clipboard)  (GtkEditable *editable);
+  void (* paste_clipboard) (GtkEditable *editable);
+
+  /* Virtual functions. get_chars is in paricular not a signal because
+   * it returns malloced memory. The others are not signals because
+   * they would not be particularly useful as such. (All changes to
+   * selection and position do not go through these functions)
+   */
   void (* update_text)  (GtkEditable    *editable,
 			 gint            start_pos,
 			 gint            end_pos);
@@ -74,11 +111,11 @@ struct _GtkEditableClass
   void (* set_selection)(GtkEditable    *editable,
 			 gint            start_pos,
 			 gint            end_pos);
-  void (* activate)     (GtkEditable    *editable);
-  void (* changed)      (GtkEditable    *editable);
+  void (* set_position) (GtkEditable    *editable,
+			 gint            position);
 };
 
-guint      gtk_editable_get_type       (void);
+GtkType    gtk_editable_get_type       (void);
 void       gtk_editable_select_region  (GtkEditable      *editable,
 					gint              start,
 					gint              end);
@@ -89,21 +126,23 @@ void       gtk_editable_insert_text   (GtkEditable       *editable,
 void       gtk_editable_delete_text    (GtkEditable      *editable,
 					gint              start_pos,
 					gint              end_pos);
-gchar *    gtk_editable_get_chars      (GtkEditable      *editable,
+gchar*     gtk_editable_get_chars      (GtkEditable      *editable,
 					gint              start_pos,
 					gint              end_pos);
-void       gtk_editable_cut_clipboard  (GtkEditable      *editable,
-					guint32           time);
-void       gtk_editable_copy_clipboard (GtkEditable      *editable, 
-					guint32           time);
-void       gtk_editable_paste_clipboard (GtkEditable     *editable, 
-					 guint32          time);
+void       gtk_editable_cut_clipboard  (GtkEditable      *editable);
+void       gtk_editable_copy_clipboard (GtkEditable      *editable);
+void       gtk_editable_paste_clipboard (GtkEditable     *editable);
 void       gtk_editable_claim_selection (GtkEditable     *editable, 
 					 gboolean         claim, 
 					 guint32          time);
 void       gtk_editable_delete_selection (GtkEditable    *editable);
 
 void       gtk_editable_changed         (GtkEditable    *editable);
+void       gtk_editable_set_position    (GtkEditable    *editable,
+					 gint            position);
+gint       gtk_editable_get_position    (GtkEditable    *editable);
+void       gtk_editable_set_editable    (GtkEditable    *editable,
+					 gboolean        is_editable);
 
 
 #ifdef __cplusplus
