@@ -24,19 +24,19 @@ static void gtk_event_box_class_init               (GtkEventBoxClass *klass);
 static void gtk_event_box_init                     (GtkEventBox      *event_box);
 static void gtk_event_box_realize                  (GtkWidget        *widget);
 static void gtk_event_box_size_request             (GtkWidget        *widget,
-						   GtkRequisition   *requisition);
+						    GtkRequisition   *requisition);
 static void gtk_event_box_size_allocate            (GtkWidget        *widget,
-						   GtkAllocation    *allocation);
+						    GtkAllocation    *allocation);
 static void gtk_event_box_draw                     (GtkWidget    *widget,
 						   GdkRectangle *area);
 static gint gtk_event_box_expose                   (GtkWidget      *widget,
 						   GdkEventExpose *event);
 
 
-guint
-gtk_event_box_get_type ()
+GtkType
+gtk_event_box_get_type (void)
 {
-  static guint event_box_type = 0;
+  static GtkType event_box_type = 0;
 
   if (!event_box_type)
     {
@@ -47,8 +47,9 @@ gtk_event_box_get_type ()
 	sizeof (GtkEventBoxClass),
 	(GtkClassInitFunc) gtk_event_box_class_init,
 	(GtkObjectInitFunc) gtk_event_box_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
       event_box_type = gtk_type_unique (gtk_bin_get_type (), &event_box_info);
@@ -79,7 +80,7 @@ gtk_event_box_init (GtkEventBox *event_box)
 }
 
 GtkWidget*
-gtk_event_box_new ()
+gtk_event_box_new (void)
 {
   return GTK_WIDGET ( gtk_type_new (gtk_event_box_get_type ()));
 }
@@ -175,7 +176,7 @@ gtk_event_box_size_allocate (GtkWidget     *widget,
 			      child_allocation.height);
     }
   
-  if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
+  if (bin->child)
     {
       gtk_widget_size_allocate (bin->child, &child_allocation);
     }
