@@ -46,7 +46,7 @@ static gint gtk_hscale_trough_keys   (GtkRange *range,
 				      GtkTroughType *pos);
 
 guint
-gtk_hscale_get_type ()
+gtk_hscale_get_type (void)
 {
   static guint hscale_type = 0;
 
@@ -59,8 +59,9 @@ gtk_hscale_get_type ()
 	sizeof (GtkHScaleClass),
 	(GtkClassInitFunc) gtk_hscale_class_init,
 	(GtkObjectInitFunc) gtk_hscale_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
       hscale_type = gtk_type_unique (gtk_scale_get_type (), &hscale_info);
