@@ -20,6 +20,7 @@
 #include "gtkaccellabel.h"
 #include "gtkmain.h"
 #include "gtkmenu.h"
+#include "gtkmenubar.h"
 #include "gtkmenuitem.h"
 #include "gtksignal.h"
 
@@ -32,6 +33,7 @@
 
 enum {
   ACTIVATE,
+  ACTIVATE_ITEM,
   LAST_SIGNAL
 };
 
@@ -51,6 +53,7 @@ static gint gtk_menu_item_expose         (GtkWidget        *widget,
 					  GdkEventExpose   *event);
 static void gtk_real_menu_item_select    (GtkItem          *item);
 static void gtk_real_menu_item_deselect  (GtkItem          *item);
+static void gtk_real_menu_item_activate_item  (GtkMenuItem      *item);
 static gint gtk_menu_item_select_timeout (gpointer          data);
 static void gtk_menu_item_position_menu  (GtkMenu          *menu,
 					  gint             *x,
@@ -58,6 +61,10 @@ static void gtk_menu_item_position_menu  (GtkMenu          *menu,
 					  gpointer          user_data);
 static void gtk_menu_item_show_all       (GtkWidget        *widget);
 static void gtk_menu_item_hide_all       (GtkWidget        *widget);
+static void gtk_menu_item_forall         (GtkContainer    *container,
+					  gboolean         include_internals,
+					  GtkCallback      callback,
+					  gpointer         callback_data);
 
 static GtkItemClass *parent_class;
 static guint menu_item_signals[LAST_SIGNAL] = { 0 };
@@ -94,10 +101,12 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
 {
   GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
+  GtkContainerClass *container_class;
   GtkItemClass *item_class;
 
   object_class = (GtkObjectClass*) klass;
   widget_class = (GtkWidgetClass*) klass;
+  container_class = (GtkContainerClass*) klass;
   item_class = (GtkItemClass*) klass;
 
   parent_class = gtk_type_class (gtk_item_get_type ());
@@ -110,6 +119,14 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
                     gtk_marshal_NONE__NONE,
 		    GTK_TYPE_NONE, 0);
 
+  menu_item_signals[ACTIVATE_ITEM] =
+    gtk_signal_new ("activate_item",
+                    GTK_RUN_FIRST,
+                    object_class->type,
+                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, activate_item),
+                    gtk_signal_default_marshaller,
+		    GTK_TYPE_NONE, 0);
+
   gtk_object_class_add_signals (object_class, menu_item_signals, LAST_SIGNAL);
 
   object_class->destroy = gtk_menu_item_destroy;
@@ -120,14 +137,18 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   widget_class->draw = gtk_menu_item_draw;
   widget_class->expose_event = gtk_menu_item_expose;
   widget_class->show_all = gtk_menu_item_show_all;
-  widget_class->hide_all = gtk_menu_item_hide_all;  
+  widget_class->hide_all = gtk_menu_item_hide_all;
+
+  container_class->forall = gtk_menu_item_forall;
 
   item_class->select = gtk_real_menu_item_select;
   item_class->deselect = gtk_real_menu_item_deselect;
 
   klass->activate = NULL;
+  klass->activate_item = gtk_real_menu_item_activate_item;
 
   klass->toggle_size = 0;
+  klass->hide_on_activate = TRUE;
 }
 
 static void
@@ -493,8 +514,16 @@ gtk_real_menu_item_select (GtkItem *item)
 
   menu_item = GTK_MENU_ITEM (item);
 
-  if (menu_item->submenu && !GTK_WIDGET_VISIBLE (menu_item->submenu))
-    menu_item->timer = gtk_timeout_add (SELECT_TIMEOUT, gtk_menu_item_select_timeout, menu_item);
+  /*  if (menu_item->submenu && !GTK_WIDGET_VISIBLE (menu_item->submenu))*/
+  if (menu_item->submenu)
+    {
+  /* Boy this is a hack! */
+      GdkEvent *current_event = gtk_get_current_event();
+      if (current_event && (current_event->type != GDK_ENTER_NOTIFY))
+	gtk_menu_item_select_timeout (menu_item);
+      else
+	menu_item->timer = gtk_timeout_add (SELECT_TIMEOUT, gtk_menu_item_select_timeout, menu_item);
+    }
 
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_PRELIGHT);
   gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
@@ -522,6 +551,38 @@ gtk_real_menu_item_deselect (GtkItem *item)
   gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
 }
 
+static void
+gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
+{
+  GtkWidget *widget;
+
+  g_return_if_fail (menu_item != NULL);
+  g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
+
+  widget = GTK_WIDGET (menu_item);
+  
+  if (widget->parent &&
+      GTK_IS_MENU_SHELL (widget->parent))
+    {
+      if (menu_item->submenu == NULL)
+	gtk_menu_shell_activate_item (GTK_MENU_SHELL (widget->parent),
+				      widget, TRUE);
+      else
+	{
+	  GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget->parent);
+
+	  if (!menu_shell->active)
+	    {
+	      gtk_grab_add (GTK_WIDGET (menu_shell));
+	      menu_shell->have_grab = TRUE;
+	      menu_shell->active = TRUE;
+	    }
+
+	  gtk_menu_shell_select_item (GTK_MENU_SHELL (widget->parent), widget);
+	}
+    }
+}
+
 static gint
 gtk_menu_item_select_timeout (gpointer data)
 {
@@ -538,6 +599,16 @@ gtk_menu_item_select_timeout (gpointer data)
 		  GTK_MENU_SHELL (GTK_WIDGET (menu_item)->parent)->button,
 		  0);
 
+  /* This is a bit of a hack - we want to select the first item
+   * of menus hanging of a menu bar, but not for cascading submenus
+   */
+  if (GTK_IS_MENU_BAR (GTK_WIDGET (menu_item)->parent))
+    {
+      GtkMenuShell *submenu = GTK_MENU_SHELL (menu_item->submenu);
+      if (submenu->children)
+	gtk_menu_shell_select_item (submenu, submenu->children->data);
+    }
+
   return FALSE;
 }
 
@@ -643,39 +714,61 @@ gtk_menu_item_right_justify(GtkMenuItem *menuitem)
   menuitem->right_justify = 1;
 }
 
+
 static void
 gtk_menu_item_show_all (GtkWidget *widget)
 {
-  GtkContainer *container;
-  GtkMenuItem  *menu_item;
+  GtkMenuItem *menu_item;
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-  container = GTK_CONTAINER (widget);
+
   menu_item = GTK_MENU_ITEM (widget);
 
-  /* Show children, traverse to submenu, show self. */
-  gtk_container_foreach (container, (GtkCallback) gtk_widget_show_all, NULL);
+  /* show children including submenu */
   if (menu_item->submenu)
     gtk_widget_show_all (menu_item->submenu);
+  gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_show_all, NULL);
+
   gtk_widget_show (widget);
 }
 
-
 static void
 gtk_menu_item_hide_all (GtkWidget *widget)
 {
-  GtkContainer *container;
-  GtkMenuItem  *menu_item;
+  GtkMenuItem *menu_item;
 
   g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
-  container = GTK_CONTAINER (widget);
+
+  gtk_widget_hide (widget);
+
   menu_item = GTK_MENU_ITEM (widget);
 
-  /* Reverse order of gtk_menu_item_show_all */
-  gtk_widget_hide (widget);
+  /* hide children including submenu */
+  gtk_container_foreach (GTK_CONTAINER (widget), (GtkCallback) gtk_widget_hide_all, NULL);
   if (menu_item->submenu)
     gtk_widget_hide_all (menu_item->submenu);
-  gtk_container_foreach (container, (GtkCallback) gtk_widget_hide_all, NULL);
+}
+
+static void
+gtk_menu_item_forall (GtkContainer *container,
+		      gboolean      include_internals,
+		      GtkCallback   callback,
+		      gpointer      callback_data)
+{
+  GtkBin *bin;
+  GtkMenuItem *menu_item;
+
+  g_return_if_fail (container != NULL);
+  g_return_if_fail (GTK_IS_MENU_ITEM (container));
+  g_return_if_fail (callback != NULL);
+
+  bin = GTK_BIN (container);
+  menu_item = GTK_MENU_ITEM (container);
+
+  if (bin->child)
+    (* callback) (bin->child, callback_data);
+  if (include_internals && menu_item->submenu)
+    (* callback) (menu_item->submenu, callback_data);
 }
