@@ -28,10 +28,10 @@ static void gtk_pixmap_finalize   (GtkObject       *object);
 
 static GtkWidgetClass *parent_class;
 
-guint
+GtkType
 gtk_pixmap_get_type (void)
 {
-  static guint pixmap_type = 0;
+  static GtkType pixmap_type = 0;
 
   if (!pixmap_type)
     {
@@ -42,11 +42,12 @@ gtk_pixmap_get_type (void)
 	sizeof (GtkPixmapClass),
 	(GtkClassInitFunc) gtk_pixmap_class_init,
 	(GtkObjectInitFunc) gtk_pixmap_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
-      pixmap_type = gtk_type_unique (gtk_misc_get_type (), &pixmap_info);
+      pixmap_type = gtk_type_unique (GTK_TYPE_MISC, &pixmap_info);
     }
 
   return pixmap_type;
@@ -73,6 +74,7 @@ gtk_pixmap_init (GtkPixmap *pixmap)
 
   pixmap->pixmap = NULL;
   pixmap->mask = NULL;
+  pixmap->needs_clear = FALSE;
 }
 
 GtkWidget*
@@ -133,6 +135,15 @@ gtk_pixmap_set (GtkPixmap *pixmap,
 	}
       if (GTK_WIDGET_VISIBLE (pixmap))
 	{
+	  /* If we aren't drawing the entire area, clear first */
+	  if (GTK_WIDGET_DRAWABLE (pixmap) && 
+	      ((mask != NULL) || 
+	       (GTK_WIDGET (pixmap)->requisition.width != oldwidth) ||
+	       (GTK_WIDGET (pixmap)->requisition.height != oldheight)))
+	    {
+	      pixmap->needs_clear = TRUE;
+	    }
+
 	  if ((GTK_WIDGET (pixmap)->requisition.width != oldwidth) ||
 	      (GTK_WIDGET (pixmap)->requisition.height != oldheight))
 	    gtk_widget_queue_resize (GTK_WIDGET (pixmap));
@@ -183,6 +194,17 @@ gtk_pixmap_expose (GtkWidget      *widget,
       pixmap = GTK_PIXMAP (widget);
       misc = GTK_MISC (widget);
 
+      if (pixmap->needs_clear)
+	{
+          gdk_window_clear_area (GTK_WIDGET (pixmap)->window,
+				 GTK_WIDGET (pixmap)->allocation.x,
+				 GTK_WIDGET (pixmap)->allocation.y,
+				 GTK_WIDGET (pixmap)->allocation.width,
+				 GTK_WIDGET (pixmap)->allocation.height);
+	  
+	  pixmap->needs_clear = FALSE;
+	}
+
       x = (widget->allocation.x * (1.0 - misc->xalign) +
 	   (widget->allocation.x + widget->allocation.width
 	    - (widget->requisition.width - misc->xpad * 2)) *
