@@ -1159,30 +1159,6 @@ gtk_signal_handlers_destroy (GtkObject *object)
     }
 }
 
-/* Work around the
-   '#define gtk_signal_default_marshaller gtk_marshal_NONE__NONE'
-   in gtkmarshal.h
-
-   This is here to provide some sort of basic backwards binary
-   compatibility.  It is not listed in gtksignal.h in order to make
-   sure people recompiling things see warnings and make changes as needed.
-   -ECL
-*/
-#undef gtk_signal_default_marshaller
-void
-gtk_signal_default_marshaller (GtkObject      *object,
-			       GtkSignalFunc   func,
-			       gpointer	       func_data,
-			       GtkArg	      *params)
-{
-  GtkSignalMarshaller0 rfunc;
-  
-  rfunc = (GtkSignalMarshaller0) func;
-  
-  (* rfunc) (object, func_data);
-}
-#define gtk_signal_default_marshaller gtk_marshal_NONE__NONE
-
 void
 gtk_signal_set_funcs (GtkSignalMarshal marshal_func,
 		      GtkSignalDestroy destroy_func)
@@ -1804,9 +1780,10 @@ gtk_signal_collect_params (GtkArg	       *params,
 
       params->type = *(param_types++);
       params->name = NULL;
-      error = gtk_arg_collect_value (GTK_FUNDAMENTAL_TYPE (params->type),
-				     params,
-				     &var_args);
+      GTK_ARG_COLLECT_VALUE (GTK_FUNDAMENTAL_TYPE (params->type),
+			     params,
+			     var_args,
+			     error);
       if (error)
 	{
 	  failed = TRUE;
