@@ -24,7 +24,6 @@
 #include <gtk/gtkadjustment.h>
 #include <gtk/gtkeditable.h>
 
-
 #ifdef __cplusplus
 extern "C" {
 #pragma }
@@ -37,7 +36,7 @@ extern "C" {
 #define GTK_IS_TEXT(obj)               (GTK_CHECK_TYPE ((obj), GTK_TYPE_TEXT))
 #define GTK_IS_TEXT_CLASS(klass)       (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TEXT))
 
-  
+typedef struct _GtkTextFont       GtkTextFont;
 typedef struct _GtkPropertyMark   GtkPropertyMark;
 typedef struct _GtkText           GtkText;
 typedef struct _GtkTextClass      GtkTextClass;
@@ -100,8 +99,6 @@ struct _GtkText
 
 			     /* FLAGS */
 
-  /* True iff the cursor has been placed yet. */
-  guint has_cursor : 1;
   /* True iff this buffer is wrapping lines, otherwise it is using a
    * horizontal scrollbar. */
   guint line_wrap : 1;
@@ -149,6 +146,8 @@ struct _GtkText
   GList *tab_stops;
   gint default_tab_width;
 
+  GtkTextFont *current_font;	/* Text font for current style */
+
   /* Timer used for auto-scrolling off ends */
   gint timer;
   
