@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 #include <string.h>
-#include "gtkobject.h"
 #include "gtktypeutils.h"
-#include "gtkcontainer.h"
 
 
 #define	TYPE_NODES_BLOCK_SIZE	(200)
@@ -53,7 +51,7 @@ struct _GtkTypeNode
     node_var = NULL; \
 }
 
-static void  gtk_type_class_init		(GtkTypeNode *node);
+static void  gtk_type_class_init		(GtkType      node_type);
 static guint gtk_type_name_hash			(const char  *key);
 static gint  gtk_type_name_compare		(const char  *a,
 						 const char  *b);
@@ -296,7 +294,11 @@ gtk_type_parent_class (GtkType type)
       if (node)
 	{
 	  if (!node->klass)
-	    gtk_type_class_init (node);
+	    {
+	      type = node->type;
+	      gtk_type_class_init (type);
+	      LOOKUP_TYPE_NODE (node, type);
+	    }
 	  
 	  return node->klass;
 	}
@@ -314,7 +316,11 @@ gtk_type_class (GtkType type)
   g_return_val_if_fail (node != NULL, NULL);
   
   if (!node->klass)
-    gtk_type_class_init (node);
+    {
+      type = node->type;
+      gtk_type_class_init (type);
+      LOOKUP_TYPE_NODE (node, type);
+    }
   
   return node->klass;
 }
@@ -323,7 +329,7 @@ gpointer
 gtk_type_new (GtkType type)
 {
   GtkTypeNode *node;
-  GtkObject *object;
+  GtkTypeObject *tobject;
   gpointer klass;
   guint i;
   
@@ -332,13 +338,14 @@ gtk_type_new (GtkType type)
   
   klass = gtk_type_class (type);
   node->chunk_alloc_locked = TRUE;
+
   if (node->mem_chunk)
     {
-      object = g_mem_chunk_alloc (node->mem_chunk);
-      memset (object, 0, node->type_info.object_size);
+      tobject = g_mem_chunk_alloc (node->mem_chunk);
+      memset (tobject, 0, node->type_info.object_size);
     }
   else
-    object = g_malloc0 (node->type_info.object_size);
+    tobject = g_malloc0 (node->type_info.object_size);
   
   /* we need to call the base classes' object_init_func for derived
    * objects with the object's ->klass field still pointing to the
@@ -352,15 +359,15 @@ gtk_type_new (GtkType type)
       LOOKUP_TYPE_NODE (pnode, node->supers[i]);
       if (pnode->type_info.object_init_func)
 	{
-	  object->klass = pnode->klass;
-	  pnode->type_info.object_init_func (object);
+	  tobject->klass = pnode->klass;
+	  pnode->type_info.object_init_func (tobject);
 	}
     }
-  object->klass = klass;
+  tobject->klass = klass;
   if (node->type_info.object_init_func)
-    node->type_info.object_init_func (object);
+    node->type_info.object_init_func (tobject);
   
-  return object;
+  return tobject;
 }
 
 void
@@ -395,23 +402,20 @@ void
 gtk_type_describe_heritage (GtkType type)
 {
   GtkTypeNode *node;
-  gboolean first;
+  gchar *is_a = "";
   
   LOOKUP_TYPE_NODE (node, type);
-  first = TRUE;
   
   while (node)
     {
-      if (first)
-	{
-	  first = FALSE;
-	  g_print ("is a ");
-	}
-      
       if (node->type_info.type_name)
-	g_print ("%s\n", node->type_info.type_name);
+	g_message ("%s%s",
+		   is_a,
+		   node->type_info.type_name);
       else
-	g_print ("<unnamed type>\n");
+	g_message ("%s<unnamed type>",
+		   is_a);
+      is_a = "is a ";
       
       LOOKUP_TYPE_NODE (node, node->parent_type);
     }
@@ -431,19 +435,23 @@ gtk_type_describe_tree (GtkType	 type,
       GList *list;
       guint old_indent;
       guint i;
+      GString *gstring;
+
+      gstring = g_string_new ("");
       
       for (i = 0; i < indent; i++)
-	g_print (" ");
+	g_string_append_c (gstring, ' ');
       
       if (node->type_info.type_name)
-	g_print ("%s", node->type_info.type_name);
+	g_string_append (gstring, node->type_info.type_name);
       else
-	g_print ("(no-name)");
+	g_string_append (gstring, "<unnamed type>");
       
       if (show_size)
-	g_print (" ( %d bytes )\n", node->type_info.object_size);
-      else
-	g_print ("\n");
+	g_string_sprintfa (gstring, " (%d bytes)", node->type_info.object_size);
+
+      g_message ("%s", gstring->str);
+      g_string_free (gstring, TRUE);
       
       old_indent = indent;
       indent += 4;
@@ -483,15 +491,22 @@ gtk_type_is_a (GtkType type,
 }
 
 static void
-gtk_type_class_init (GtkTypeNode *node)
+gtk_type_class_init (GtkType type)
 {
+  GtkTypeNode *node;
+
+  /* we need to relookup nodes everytime we called an external function */
+  LOOKUP_TYPE_NODE (node, type);
+  
   if (!node->klass && node->type_info.class_size)
     {
-      GtkObjectClass *object_class;
+      GtkTypeClass *type_class;
       GtkTypeNode *base_node;
       GSList *slist;
       
-      g_assert (node->type_info.class_size >= sizeof (GtkObjectClass));
+      if (node->type_info.class_size < sizeof (GtkTypeClass))
+	g_warning ("The `%s' class is too small to inherit from GtkTypeClass",
+		   node->type_info.type_name);
       
       node->klass = g_malloc0 (node->type_info.class_size);
       
@@ -500,15 +515,25 @@ gtk_type_class_init (GtkTypeNode *node)
 	  GtkTypeNode *parent;
 	  
 	  LOOKUP_TYPE_NODE (parent, node->parent_type);
+	  
+	  if (node->type_info.class_size < parent->type_info.class_size)
+	    g_warning ("The `%s' class is smaller than its parent class `%s'",
+		       node->type_info.type_name,
+		       node->type_info.type_name);
+	  
 	  if (!parent->klass)
-	    gtk_type_class_init (parent);
+	    {
+	      gtk_type_class_init (parent->type);
+	      LOOKUP_TYPE_NODE (node, type);
+	      LOOKUP_TYPE_NODE (parent, node->parent_type);
+	    }
 	  
 	  if (parent->klass)
 	    memcpy (node->klass, parent->klass, parent->type_info.class_size);
 	}
       
-      object_class = node->klass;
-      object_class->type = node->type;
+      type_class = node->klass;
+      type_class->type = node->type;
       
       /* stack all base class initialization functions, so we
        * call them in ascending order.
@@ -531,6 +556,7 @@ gtk_type_class_init (GtkTypeNode *node)
 	      
 	      base_class_init = walk->data;
 	      base_class_init (node->klass);
+	      LOOKUP_TYPE_NODE (node, type);
 	    }
 	  g_slist_free (slist);
 	}
@@ -540,6 +566,86 @@ gtk_type_class_init (GtkTypeNode *node)
     }
 }
 
+static inline gchar*
+gtk_type_descriptive_name (GtkType type)
+{
+  gchar *name;
+
+  name = gtk_type_name (type);
+  if (!name)
+    name = "(unknown)";
+
+  return name;
+}
+
+GtkTypeObject*
+gtk_type_check_object_cast (GtkTypeObject  *type_object,
+			    GtkType         cast_type)
+{
+  if (!type_object)
+    {
+      g_warning ("invalid cast from (NULL) pointer to `%s'",
+		 gtk_type_descriptive_name (cast_type));
+      return type_object;
+    }
+  if (!type_object->klass)
+    {
+      g_warning ("invalid unclassed pointer in cast to `%s'",
+		 gtk_type_descriptive_name (cast_type));
+      return type_object;
+    }
+  /* currently, GTK_TYPE_OBJECT is the lowest fundamental type
+   * dominator for types that introduce classes.
+   */
+  if (type_object->klass->type < GTK_TYPE_OBJECT)
+    {
+      g_warning ("invalid class type `%s' in cast to `%s'",
+		 gtk_type_descriptive_name (type_object->klass->type),
+		 gtk_type_descriptive_name (cast_type));
+      return type_object;
+    }
+  if (!gtk_type_is_a (type_object->klass->type, cast_type))
+    {
+      g_warning ("invalid cast from `%s' to `%s'",
+		 gtk_type_descriptive_name (type_object->klass->type),
+		 gtk_type_descriptive_name (cast_type));
+      return type_object;
+    }
+
+  return type_object;
+}
+
+GtkTypeClass*
+gtk_type_check_class_cast (GtkTypeClass   *klass,
+			   GtkType         cast_type)
+{
+  if (!klass)
+    {
+      g_warning ("invalid class cast from (NULL) pointer to `%s'",
+		 gtk_type_descriptive_name (cast_type));
+      return klass;
+    }
+  /* currently, GTK_TYPE_OBJECT is the lowest fundamental type
+   * dominator for types that introduce classes.
+   */
+  if (klass->type < GTK_TYPE_OBJECT)
+    {
+      g_warning ("invalid class type `%s' in class cast to `%s'",
+		 gtk_type_descriptive_name (klass->type),
+		 gtk_type_descriptive_name (cast_type));
+      return klass;
+    }
+  if (!gtk_type_is_a (klass->type, cast_type))
+    {
+      g_warning ("invalid class cast from `%s' to `%s'",
+		 gtk_type_descriptive_name (klass->type),
+		 gtk_type_descriptive_name (cast_type));
+      return klass;
+    }
+
+  return klass;
+}
+
 GtkEnumValue*
 gtk_type_enum_get_values (GtkType      enum_type)
 {
