@@ -43,8 +43,9 @@ gtk_vbutton_box_get_type (void)
 	sizeof (GtkVButtonBoxClass),
 	(GtkClassInitFunc) gtk_vbutton_box_class_init,
 	(GtkObjectInitFunc) gtk_vbutton_box_init,
-	(GtkArgSetFunc) NULL,
-        (GtkArgGetFunc) NULL,
+	/* reserved_1 */ NULL,
+        /* reserved_2 */ NULL,
+        (GtkClassInitFunc) NULL,
       };
 
       vbutton_box_type = gtk_type_unique (gtk_button_box_get_type (), &vbutton_box_info);
@@ -242,7 +243,7 @@ gtk_vbutton_box_size_allocate (GtkWidget     *widget,
     break;
   case GTK_BUTTONBOX_END:
     childspacing = spacing;
-    y = allocation->x + allocation->height - child_height * nvis_children
+    y = allocation->y + allocation->height - child_height * nvis_children
 	    - spacing * (nvis_children-1)
 	    - GTK_CONTAINER (box)->border_width;
     break;
